<?php

namespace FiloBlu\Rma\Plugin\Model;

use Exception;
use FiloBlu\Rma\Model\Rma\Status\History;
use FiloBlu\Rma\Model\Rma\Status\HistoryFactory;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Rma\Model\Rma\Source\Status;
use Magento\Store\Model\ScopeInterface;
use Psr\Log\LoggerInterface;
use Magento\Rma\Model\Item\Attribute\Source\Status as ItemStatus;
use FiloBlu\Rma\Helper\SerializerHelper;
use Magento\Rma\Api\RmaRepositoryInterface;
use FiloBlu\Rma\Api\Model\Config\Reader;
/**
 * Class Rma
 * @package FiloBlu\Rma\Plugin\Model
 */
class Rma
{
    /**
     * @var string
     */
    const XML_PATH_RECEIVED_EMAIL_ENABLE = 'sales_email/magento_rma_receive/enabled';

    /**
     * @var string
     */
    const XML_PATH_RMA_REASON_MAPPING = 'sales/rma_reason/reason';

    /**
     * @var HistoryFactory
     */
    protected $historyFactory;

    /**
     * @var LoggerInterface
     */
    protected $logger;

    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var SerializerHelper
     */
    protected $serializer;

    /**
     * Rma constructor.
     * @param HistoryFactory $historyFactory
     * @param ScopeConfigInterface $scopeConfig
     * @param RmaRepositoryInterface $rmaRepository
     * @param Reader $reader
     * @param LoggerInterface $logger
     * @param SerializerHelper $serializer
     */
    public function __construct(
        HistoryFactory $historyFactory,
        ScopeConfigInterface $scopeConfig,
        RmaRepositoryInterface $rmaRepository,
        Reader $reader,
        LoggerInterface $logger,
        SerializerHelper $serializer)
    {
        $this->historyFactory = $historyFactory;
        $this->scopeConfig = $scopeConfig;
        $this->rmaRepository = $rmaRepository;
        $this->reader = $reader;
        $this->logger = $logger;
        $this->serializer = $serializer;
    }

    /**
     * @param \Magento\Rma\Model\Rma $subject
     * @param $result
     * @return \Magento\Rma\Model\Rma
     */
    public function afterSaveRma(\Magento\Rma\Model\Rma $subject, $result)
    {
        if ($result === false) {
            return $result;
        }

        try {

            $storeId = $subject->getStoreId();
            $status = $result->getStatus();

            switch ($status) {
                case  Status::STATE_RECEIVED:
                    
                    /* change order status to payment_confirmed for change size resolution */
                    

                    if (!boolval($this->getConfigValue(self::XML_PATH_RECEIVED_EMAIL_ENABLE, $storeId))) {
                        return $result;
                    }

                    /** @var $result \Magento\Rma\Model\Rma */
                    /** @var $statusHistory History */
                    $statusHistory = $this->historyFactory->create();
                    $statusHistory->setRmaEntityId($result->getEntityId());
                    $statusHistory->sendReceivedEmail();
                    
                    break;

                case  Status::STATE_PENDING:

                    /** @var $result \Magento\Rma\Model\Rma */
                    $items = $result->getItems();
                    $standard_reasons_config = $this->reader->read();
                    $automatic_approval = array();
                    foreach($standard_reasons_config as $reason):
                        if($reason['automatic_approval']==1):
                            $automatic_approval[] = $reason['id'];
                        endif;
                    endforeach;
                    /** @var \Magento\Rma\Api\Data\ItemInterface $item */
                    $auth = 0;

                    $itemsComments ="";
                    foreach($items as $item):
                        $item_status = $item->getStatus();
                        $item_brand_reason = $item->getReason();
                        $mapping = $this->serializer->unserialize($this->getConfigValue(self::XML_PATH_RMA_REASON_MAPPING, $storeId));
                        foreach ($mapping as $reason):
                            if($item_brand_reason == $reason['brand_reason'] && in_array($reason['standard_reason'],$automatic_approval)):
                                // avanzamento dello stato dell'item
                                $item->setStatus(ItemStatus::STATE_AUTHORIZED);
                                $itemsComments  .=" -- ".$item->getProductAdminName()." automatically authorized -- ";
                                $item->setQtyAuthorized($item->getQtyRequested());
                                $item->save();
                                $auth++;
                                break;
                            endif;
                        endforeach;
                    endforeach;

                    if($auth > 0 && $auth==count($items)):
                        $this->rmaRepository->get($result->getEntityId())
                            ->setStatus(Status::STATE_AUTHORIZED)
                            ->save();
                    elseif($auth > 0):
                        $this->rmaRepository->get($result->getEntityId())
                            ->setStatus(Status::STATE_PARTIAL_AUTHORIZED)
                            ->save();
                    endif;

                    if($auth > 0){
                        $statusHistory = $this->historyFactory->create();
                        $statusHistory->setRmaEntityId($result->getEntityId());
                        $statusHistory->saveComment($itemsComments,false,true);
                    }

                    break;
            }

        } catch (Exception $e) {
            $this->logger->critical($e);
            return $result;
        }

        return $result;
    }

    /**
     * Return store configuration value
     *
     * @param string $path
     * @param int $storeId
     * @return mixed
     */
    protected function getConfigValue($path, $storeId)
    {
        return $this->scopeConfig->getValue(
            $path,
            ScopeInterface::SCOPE_STORE,
            $storeId
        );
    }

}

